package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Tween;
	import net.flashpunk.tweens.misc.Alarm;
	import net.flashpunk.utils.Draw;

	public class Planet extends SpaceThing
	{
		[Embed(source = "../assets/planetA.png")]
		static public const ImgA:Class;
		[Embed(source = "../assets/planetAshadow.png")]
		static public const ImgAShadow:Class;
		[Embed(source = "../assets/arrow.png")]
		static public const ImgArrow:Class;
		[Embed(source = "../assets/planetTrail.png")]
		static public const ImgTrail:Class;
		
		private const MAX_SPEED:Number = 7;
		private const TRAIL:int = 10;
		private const GRAV2:Number = 0.04;
		
		private var image:Image;
		private var arrow:Spritemap;
		private var shadow:Image;
		public var speed:Point = new Point;
		private var sun:Sun;
		private var other:Sun;
		private var toCollide:Vector.<SpaceThing> = new Vector.<SpaceThing>;
		private var collisions:Vector.<SpaceThing> = new Vector.<SpaceThing>;
		private var oldCollisions:Vector.<SpaceThing> = new Vector.<SpaceThing>;
		private var canHitSun:Boolean = true;
		private var canHitAlarm:Alarm;
		
		private var trail:Vector.<Image> = new Vector.<Image>;
		
		public function Planet(x:int, y:int, sun:Sun, other:Sun, color:uint, img:Class, shadowImg:Class) 
		{
			super(x, y, 16);
			this.sun = sun;
			this.other = other;
			type = "planet";
			
			for (var i:int = 0; i < TRAIL; i++)
			{
				trail.push(new Image(ImgTrail));
				trail[i].centerOrigin();
				trail[i].alpha = .3 - (i / Number(TRAIL)) * .3;
				trail[i].relative = false;
				trail[i].x = x;
				trail[i].y = y;
				trail[i].color = color;
				addGraphic(trail[i]);
			}
			
			image = new Image(img);
			image.centerOrigin();
			image.color = color;
			addGraphic(image);
			
			shadow = new Image(shadowImg);
			shadow.centerOrigin();
			shadow.alpha = .5;
			addGraphic(shadow);
			
			arrow = new Spritemap(ImgArrow, 24, 24);
			arrow.add("a", [1, 0, 1], .08);
			arrow.play("a");
			arrow.color = color;
			arrow.centerOrigin();
			arrow.visible = arrow.relative = false;
			addGraphic(arrow);
			
			setHitbox(image.width * .8, image.height * .8);
			centerOrigin();
			
			handleShadow();
			
			canHitAlarm = new Alarm(60, canHit, Tween.PERSIST);
			addTween(canHitAlarm);
			
			FP.angleXY(speed, FP.angle(x, y, sun.x, sun.y) - 90, 2);
		}
		
		private function canHit():void
		{
			canHitSun = true;
		}
		
		override public function added():void 
		{
			super.added();
			
			if (sun != level.sunA)
				toCollide.push(level.sunA);
			if (sun != level.sunB)
				toCollide.push(level.sunB);
			if (this != level.planetA1)
				toCollide.push(level.planetA1);
			if (this != level.planetA2)
				toCollide.push(level.planetA2);
			if (this != level.planetA3)
				toCollide.push(level.planetA3);
			if (this != level.planetB1)
				toCollide.push(level.planetB1);
			if (this != level.planetB2)
				toCollide.push(level.planetB2);
			if (this != level.planetB3)
				toCollide.push(level.planetB3);
		}
		
		public function handleShadow():void
		{
			shadow.angle = FP.angle(sun.x, sun.y, x, y);
		}
		
		override public function update():void 
		{
			super.update();
			
			//Update trail
			var i:int;
			for (i = TRAIL - 1; i > 0; i--)
			{
				trail[i].x = trail[i - 1].x;
				trail[i].y = trail[i - 1].y;
			}
			trail[0].x = x;
			trail[0].y = y;
			
			//Update physics
			FP.angleXY(FP.point, FP.angle(x, y, sun.x, sun.y), Main.GRAVITY);
			speed.x += FP.point.x;
			speed.y += FP.point.y;
			
			if (FP.distance(x, y, other.x, other.y) < 250)
			FP.angleXY(FP.point, FP.angle(x, y, other.x, other.y), GRAV2);
			speed.x += FP.point.x;
			speed.y += FP.point.y;
			
			if (speed.length > MAX_SPEED)
				speed.normalize(MAX_SPEED);
			
			x += speed.x;
			y += speed.y;
			image.angle -= 2;
			handleShadow();
			
			//Clear old collisions
			for (i = 0; i < oldCollisions.length; i++)
			{
				if (!collideThing(oldCollisions[i], x, y) && (canHitSun || oldCollisions[i] is Planet))
				{
					oldCollisions.splice(i, 1);
					i--;
				}
			}
			
			//Collide with other planets and stars
			collideOutInto(toCollide, collisions);
			for (i = 0; i < collisions.length; i++)
			{
				if (oldCollisions.indexOf(collisions[i]) == -1)
				{
					if (collisions[i] is Planet)
						(collisions[i] as Planet).hitPlanet(this);
					else if (canHitSun)
					{
						canHitSun = false;
						canHitAlarm.start();
						(collisions[i] as Sun).hit(this);
					}
					oldCollisions.push(collisions[i]);
				}
			}
		}
		
		public function hitPlanet(planet:Planet):void
		{
			if (planet.sun != sun)
				Main.SndPlanetHit.play();
			
			FP.angleXY(FP.point, FP.angle(x, y, planet.x, planet.y), 2);
			
			speed.x -= FP.point.x;
			speed.y -= FP.point.y;
			
			planet.speed.x += FP.point.x;
			planet.speed.y += FP.point.y;
			
			FP.point.normalize(radius * 2);
			x = planet.x - FP.point.x;
			y = planet.y - FP.point.y;
			
			oldCollisions.push(planet);
		}
		
		override public function render():void 
		{
			super.render();
			
			if (x > 800 || x < 0 || y < 0 || y > 600)
			{
				FP.point2.x = x - 400;
				FP.point2.y = y - 300;
				FP.point2.normalize(5);
				
				FP.point.x = 400;
				FP.point.y = 300;
				while (FP.point.x + FP.point2.x < 790 && FP.point.x + FP.point2.x > 10 && FP.point.y + FP.point2.y > 10 && FP.point.y + FP.point2.y < 590)
				{
					FP.point.x += FP.point2.x;
					FP.point.y += FP.point2.y;
				}
				
				arrow.visible = true;
				arrow.x = FP.point.x;
				arrow.y = FP.point.y;
				
				if (arrow.y < 20)
				{
					arrow.y = 16;
					arrow.angle = 90;
				}
				else if (arrow.x < 20)
				{
					arrow.x = 16;
					arrow.angle = 180;
				}
				else if (arrow.y > 580)
				{
					arrow.y = 584;
					arrow.angle = 270;
				}
				else
				{
					arrow.x = 784;
					arrow.angle = 0;
				}
			}
			else
				arrow.visible = false;
		}
		
		public function explode():void
		{
			for (var i:int = 0; i < 12; i++)
				Main.particles.emit(sun.iExplode + "B", x - 18 + FP.rand(20), y - 18 + FP.rand(20));
				
			for (i = 0; i < 4; i++)
				world.add(new Explosion(x - 20 + FP.rand(40), y - 20 + FP.rand(40), image.color));
			
			active = visible = false;
			y -= 200000;
		}
	}

}