package  
{
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Tween;
	import net.flashpunk.tweens.misc.Alarm;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.utils.Ease;
	import net.flashpunk.utils.Input;
	
	public class Sun extends SpaceThing 
	{
		[Embed(source = "../assets/sun.png")]
		static private const ImgSun:Class;
		[Embed(source = "../assets/faces.png")]
		static private const ImgFaces:Class;
		
		static private const SPEED:Number = 1.4;
		static private const ACCEL:Number = 0.1;
		static private const ADD_SPEED:Number = 1.4;
		
		private var image:Image;
		private var face:Spritemap;
		private var playerIndex:int;
		
		private var iLeft:String;
		private var iRight:String;
		private var iUp:String;
		private var iDown:String;
		private var iLava:String;
		public var iExplode:String;
		
		private var speed:Point = new Point;
		private var other:Sun;
		private var lavaAlarm:Alarm;
		private var shock:Boolean = false;
		private var faceTarget:Point = new Point;
		
		public var health:Number = 100;
		private var faceHitCounter:int = 0;
		
		private var counter:Number = 0;
		
		public function Sun(x:int, y:int, playerIndex:int) 
		{
			super(x, y, 29);
			this.playerIndex = playerIndex;
			
			type = "sun";
			
			image = new Image(ImgSun);
			image.centerOrigin();
			addGraphic(image);
			
			face = new Spritemap(ImgFaces, 24, 24, anim);
			face.centerOrigin();
			face.add("normal", [0]);
			face.add("hurt", [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1], .1, false);
			face.add("hurtWeak", [1, 1, 1, 1, 1, 1, 1, 1], .1, false);
			face.add("laugh", [2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4], .1, false);
			face.add("dead", [3]);
			face.add("sad", [5, 6], .1);
			face.play("normal");
			addGraphic(face);
			
			if (playerIndex == 0)
				image.color = 0xFFFF00;
			else
				image.color = 0x00FFFF;
			
			{
				var s:String = "p" + (playerIndex + 1);
				iLeft = s + "left";
				iRight = s + "right";
				iUp = s + "up";
				iDown = s + "down";
				iLava = s + "lava";
				iExplode = s + "explode";
			}
			
			lavaAlarm = new Alarm(6 + FP.rand(6), lava, Tween.PERSIST);
			addTween(lavaAlarm, true);
		}
		
		private function anim():void
		{
			if (face.currentAnim != "dead") 
			{ 
				if (health < 20) 
					face.play("sad"); 
				else 
				face.play("normal"); 
			}
		}
		
		private function lava():void
		{
			lavaAlarm.reset(6 + FP.rand(6));
			lavaAlarm.start();
			
			var angle:Number = FP.rand(360);
			Main.particles.setMotion(iLava, angle, 12, 50, 0, 4, 6, Ease.cubeIn);			
			FP.angleXY(FP.point, angle, radius, x, y);			
			Main.particles.emit(iLava, FP.point.x - 8, FP.point.y - 8);
		}
		
		override public function added():void 
		{
			super.added();
			
			if (playerIndex == 0)
				other = level.sunB;
			else
				other = level.sunA;
		}
		
		override public function update():void 
		{
			super.update();
			
			counter = (counter + Math.PI / 64) % (Math.PI * 2);
			image.scaleX = 1 + Math.sin(counter) * .1;
			image.scaleY = 1 + Math.sin(counter * 2) * .1;
			
			if (!shock)
			{				
				//Get input
				FP.point.x = FP.point.y = 0;
				if (level.start)
				{
					if (Input.check(iLeft))
						FP.point.x--;
					if (Input.check(iRight))
						FP.point.x++;
					if (Input.check(iUp))
						FP.point.y--;
					if (Input.check(iDown))
						FP.point.y++;
					if (FP.point.x != 0 && FP.point.y != 0)
						FP.point.normalize(1);
				}
				
				faceTarget.x = FP.point.x * 10;
				faceTarget.y = FP.point.y * 10;
				
				var spd:Number = SPEED + ADD_SPEED * (100 - health) / 100;
					
				speed.x = FP.approach(speed.x, FP.point.x * spd, ACCEL); 
				speed.y = FP.approach(speed.y, FP.point.y * spd, ACCEL); 
			}
			else
			{
				speed.normalize(speed.length * .93);
				Main.particles.emit(iExplode + "B", x - 30 + FP.rand(60), y - 30 + FP.rand(60));
			}
			
			x += speed.x;
			y += speed.y;
			
			if (faceHitCounter <= 0)
			{
				if (face.x != faceTarget.x || face.y != faceTarget.y)
				{				
					FP.point.x = Math.abs(faceTarget.x);
					FP.point.y = Math.abs(faceTarget.y);
					FP.point.normalize(0.8);
					face.x = FP.approach(face.x, faceTarget.x, FP.point.x);
					face.y = FP.approach(face.y, faceTarget.y, FP.point.y);
				}	
			}
			else
				faceHitCounter--;
			
			if (x < 30)
			{
				x = 30;
				speed.x = 0;
			}
			else if (x > 770)
			{
				x = 770;
				speed.x = 0;
			}
			
			if (y < 70)
			{
				y = 70;
				speed.y = 0;
			}
			else if (y > 570)
			{
				y = 570;
				speed.y = 0;
			}
		}
		
		private function explode():void
		{
			Main.SndExplode.play();
			level.shake = 40;
			
			var i:int;
			for (i = 0; i < 10; i++)
				Main.particles.emit(iExplode, x - 68 + FP.rand(120), y - 68 + FP.rand(120));
				
			for (i = 0; i < 12; i++)
				Main.particles.emit(iExplode + "B", x - 38 + FP.rand(60), y - 38 + FP.rand(60));
				
			if (!level.win)
			{
				level.win = true;
				world.add(new Win(playerIndex == 1 ? 0 : 1));
			}
			
			level.screenFlash(image.color);
			
			for (i = 0; i < 10; i++)
				world.add(new Explosion(x - 40 + FP.rand(80), y - 40 + FP.rand(80), image.color));
			
			active = visible = false;
			y -= 200000;
		}
		
		private function flash():void
		{
			visible = !visible;
		}
		
		public function laugh():void
		{
			if (!face.currentAnim != "dead")
				face.play("laugh");
		}
		
		public function hit(planet:Planet):void
		{
			var strength = planet.speed.length * 3.2;
			
			health -= strength;
			
			FP.angleXY(FP.point, FP.angle(x, y, planet.x, planet.y), planet.speed.length);
			
			speed.x -= FP.point.x;
			speed.y -= FP.point.y;
			
			/*
			FP.point.normalize(2);
			
			planet.speed.x += FP.point.x;
			planet.speed.y += FP.point.y;
			*/
			
			if (!shock)
			{		
				if (strength >= 14)
				{
					Main.SndStrongHit.play();					
					level.shake = 30;				
					if (this == level.sunA)
						level.sunB.laugh();
					else
						level.sunA.laugh();
						
					level.bg.setColor(this == level.sunA ? 0x00FFFF : 0xFFFF00);
					face.play("hurt");
				}
				else
				{
					Main.SndWeakHit.play();
					level.shake = 12;
					face.play("hurtWeak");
				}
				
				
				faceHitCounter = 20;
				FP.angleXY(face, FP.angle(planet.x, planet.y, x, y), Math.min(16, strength));
					
				if (health <= 0)
				{
					Main.SndKill.play();
					Main.SndKnockout.play(6);
					level.shake = 60;
					
					if (this == level.sunA)
					{
						level.planetA1.explode();
						level.planetA2.explode();
						level.planetA3.explode();
					}
					else
					{
						level.planetB1.explode();
						level.planetB2.explode();
						level.planetB3.explode();
					}
					
					shock = true;
					addTween(new Alarm(4, flash, Tween.LOOPING), true);
					addTween(new Alarm(90, explode, Tween.ONESHOT), true);
					
					face.play("dead");
				}			
			}
		}
	}

}